import os
from cStringIO import StringIO

from uvc.path import path
from uvc import commands, hg, main, exc
from uvc.tests.util import test_context
from uvc.tests.mock import patch

StatusOutput = commands.StatusOutput

dialect = hg.HgDialect()

topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "..", 
                        "testfiles"))

context = None

def setup_module(module):
    global context
    if not os.path.exists(topdir):
        os.mkdir(topdir)
    context = main.Context(topdir)

def test_clone_command_conversion():
    generic_clone = commands.clone(context, ["http://hg.mozilla.org/labs/bespin", 
                                    "bespin"])
    hg_clone = hg.clone(generic_clone)
    assert str(hg_clone) == "clone http://hg.mozilla.org/labs/bespin bespin"
    
def test_convert_function():
    generic_clone = commands.clone(context, ["http://hg.mozilla.org/labs/bespin", 
                                    "bespin"])
    result = dialect.convert(generic_clone)
    assert str(result) == "clone http://hg.mozilla.org/labs/bespin bespin"
    
def test_convert_unknown_command():
    class Foo(object):
        pass
    
    try:
        result = dialect.convert(Foo())
        assert False, "expected HgError for unknown command"
    except hg.HgError:
        pass

def test_commit_command():
    test_context.user = "Zaphod Beeblebrox <zaphod@onecooldude.us>"
    generic_commit = commands.commit(test_context, ["-m", "test message", "foo", "bar"])
    result = dialect.convert(generic_commit)
    assert not result.reads_remote
    assert not result.writes_remote
    command_parts = result.command_parts()
    assert command_parts == ["commit", "-u", "Zaphod Beeblebrox <zaphod@onecooldude.us>",
                            "-m", "test message", "foo", "bar"]
    
def test_diff_command():
    generic_diff = commands.diff(context, [])
    result = dialect.convert(generic_diff)
    assert not result.reads_remote
    assert not result.writes_remote
    assert str(result) == "diff"
    
def test_clone_command_simple_ssh_auth():
    context = main.Context(topdir, auth=dict(type="ssh", key="/tmp/id.rsa"))
    generic_clone = commands.clone(context, ["ssh://hg.mozilla.org/bar"])
    result = dialect.convert(generic_clone)
    assert result.reads_remote
    assert not result.writes_remote
    assert str(result) == "clone -e ssh -i /tmp/id.rsa -o StrictHostKeyChecking=no ssh://hg.mozilla.org/bar bar"
    
def test_clone_command_ssh_username_auth():
    context = main.Context(topdir, auth=dict(type="ssh", key="/tmp/id.rsa", 
                                username="someone_else"))
    generic_clone = commands.clone(context, ["ssh://hg.mozilla.org/bar"])
    result = dialect.convert(generic_clone)
    assert str(result) == "clone -e ssh -i /tmp/id.rsa -o StrictHostKeyChecking=no ssh://someone_else@hg.mozilla.org/bar bar"
    
def test_clone_command_http_password_auth():
    context = main.Context(topdir, auth=dict(type="password", 
                        username="supercooluser", password="hithere"))
    generic_clone = commands.clone(context, ["http://hg.mozilla.org/bar"])
    result = dialect.convert(generic_clone)
    assert str(result) == "clone http://supercooluser:hithere@hg.mozilla.org/bar bar"
    
def test_push_command_ssh_username_auth():
    context = main.Context(topdir, auth=dict(type="ssh", key="/tmp/id.rsa", 
                                username="someone_else"))
    generic_push = commands.push(context, ["ssh://hg.mozilla.org/bar"])
    result = dialect.convert(generic_push)
    assert result.writes_remote
    assert result.reads_remote
    assert str(result) == "push -e ssh -i /tmp/id.rsa -o StrictHostKeyChecking=no ssh://someone_else@hg.mozilla.org/bar"

def test_update_command():
    generic_update = commands.update(context, [])
    update = hg.update(generic_update)
    assert update.reads_remote
    assert not update.writes_remote
    assert update.get_command_line() == ["hg", "fetch"]
    
def test_resolved_command():
    myfile = path(topdir) / "myfile"
    myfile.write_bytes("test data")
    try:
        generic_resolved = commands.resolved(context, ["myfile"])
        resolved = hg.resolved(generic_resolved)
    finally:
        myfile.unlink()
        
    assert not resolved.reads_remote
    assert not resolved.writes_remote
    assert resolved.get_command_line() == ["hg", "resolve", "-m", "myfile"]
    
def test_resolved_command_no_targets():
    myfile = path(topdir) / "myfile"
    myfile.write_bytes("test data")
    try:
        generic_resolved = commands.resolved(context, [])
        resolved = hg.resolved(generic_resolved)
    finally:
        myfile.unlink()
        
    assert not resolved.reads_remote
    assert not resolved.writes_remote
    assert resolved.get_command_line() == ["hg", "resolve", "-m", "-a"]
    
def test_remove_command_no_targets():
    myfile = path(topdir) / "myfile"
    myfile.write_bytes("test data")
    try:
        generic_remove = commands.remove(context, ["myfile"])
        remove = hg.remove(generic_remove)
    finally:
        myfile.unlink()
        
    assert not remove.reads_remote
    assert not remove.writes_remote
    assert remove.get_command_line() == ["hg", "remove", "-f", "myfile"]
    
def test_status_command():
    generic_status = commands.status(context, [])
    status = hg.status(generic_status)
    assert not status.writes_remote
    assert not status.reads_remote
    
    assert status.get_command_line() == ["hg", "status"]
    
    status_output = """M modified/file
A new/file
R removed/file
C clean/file
! missing/file
? unknown/file
I ignored/file
"""
    
    output = status.process_output(0, StringIO(status_output))
    the_list = output.as_list()
    print the_list
    assert the_list == [
[StatusOutput.MODIFIED, "modified/file"],
[StatusOutput.ADDED, "new/file"],
[StatusOutput.REMOVED, "removed/file"],
[StatusOutput.CLEAN, "clean/file"],
[StatusOutput.MISSING, "missing/file"],
[StatusOutput.UNKNOWN, "unknown/file"],
[StatusOutput.IGNORED, "ignored/file"]
]
    assert str(output) == status_output
    
@patch("uvc.main.infer_dialect")
def test_init_command(infer_dialect):
    infer_dialect.return_value = None
    init = hg.init.from_args(test_context, [])
    assert init.command_parts() == ["init"]
    assert init.get_command_line() == ["hg", "init"]
    
@patch('uvc.main.infer_dialect')
def test_init_command_in_existing_repo(infer_dialect):
    infer_dialect.return_value = dialect
    
    try:
        init = hg.init.from_args(test_context, [])
        assert False, "Should have gotten an exception about existing repo"
    except exc.RepositoryAlreadyInitialized:
        pass
    
def test_revert_command():
    myfile = path(topdir) / "myfile"
    myfile.write_bytes("test data")
    try:
        generic_revert = commands.revert(context, [])
        revert = hg.revert(generic_revert)
    finally:
        myfile.unlink()
        
    assert not revert.reads_remote
    assert not revert.writes_remote
    assert revert.get_command_line() == ["hg", "revert", "--no-backup", "-a"]
    



from uvc.path import path
from uvc import commands
from uvc.tests.util import test_context
from uvc.main import Context, SecureContext
from uvc.exc import UVCError

topdir = (path(__file__).parent.parent.parent) / "testfiles"

context = None
secure_context = None

def setup_module(module):
    global context, secure_context
    if not topdir.exists:
        topdir.mkdir()
    context = Context(topdir)
    secure_context = SecureContext(topdir)
    
def test_create_clone_command():
    clone = commands.clone(context, ["http://hg.mozilla.org/labs/bespin", 
                                    "bespin"])
    assert clone.reads_remote
    assert not clone.writes_remote
    assert clone.source == "http://hg.mozilla.org/labs/bespin"
    assert clone.dest == "bespin"
    assert str(clone) == "clone http://hg.mozilla.org/labs/bespin bespin"
    
def test_bad_clone_command():
    try:
        clone = commands.clone(context, [])
        assert False, "Expected BadArgument exception for missing args."
    except commands.BadArgument:
        pass
    
def test_commit_command():
    commit = commands.commit(test_context, 
            ["-m", "my commit message", "bespin", "uvc"])
    assert not commit.reads_remote
    assert not commit.writes_remote
    message = commit.message
    assert message == "my commit message"
    assert commit.targets == ["bespin", "uvc"]
    assert str(commit) == "commit -m my commit message bespin uvc"
    
    try:
        commit = commands.commit(test_context, ["foo"])
        assert False, "Expected prompting for value"
    except commands.GetValueFromEditor,e :
        assert e.template_args == ["commit", '-m', commands.inserted_value, "foo"]
        assert e.prompt == "Please enter a commit message"
    
def test_diff_command():
    diff = commands.diff(test_context, ["myfile"])
    assert not diff.reads_remote
    assert not diff.writes_remote
    assert diff.targets == ["myfile"]
    assert str(diff) == "diff myfile"
    
    diff = commands.diff(test_context, [])
    assert diff.targets == None
    assert str(diff) == "diff"
    
def test_status_command():
    status = commands.status(test_context, [])
    assert not status.reads_remote
    assert not status.writes_remote
    assert str(status) == "status"
    
    status = commands.status(test_context, ["mydir"])
    assert str(status) == "status mydir"

def test_remove_command():
    remove = commands.remove(test_context, ["myfile"])
    assert not remove.reads_remote
    assert not remove.writes_remote
    assert remove.targets == ["myfile"]
    assert str(remove) == "remove myfile"
    
    try:
        remove = commands.remove(test_context, [])
        assert False, "Expected an error for remove without a file"
    except UVCError:
        pass

def test_revert_command():
    revert = commands.revert(test_context, ["myfile"])
    assert not revert.reads_remote
    assert not revert.writes_remote
    assert revert.targets == ["myfile"]
    assert str(revert) == "revert myfile"
    
    revert = commands.revert(test_context, [])
    assert revert.targets == None
    assert str(revert) == "revert"


def test_push_command():
    push = commands.push(test_context, [])
    assert push.reads_remote
    assert push.writes_remote
    assert str(push) == "push"
    
def test_update_command():
    update = commands.update(test_context, [])
    assert update.reads_remote
    assert not update.writes_remote
    assert str(update) == "update"

def test_update_command_with_url():
    update = commands.update(test_context, ["http://hg.mozilla.org/labs/bespin"])
    assert update.source == "http://hg.mozilla.org/labs/bespin"
    assert str(update) == "update http://hg.mozilla.org/labs/bespin"

def test_resolved_command():
    resolved = commands.resolved(test_context, ["myfile"])
    assert not resolved.reads_remote
    assert not resolved.writes_remote
    assert resolved.targets == ["myfile"]
    assert str(resolved) == "resolved myfile"
    
def test_add_command():
    add = commands.add(test_context, [])
    assert str(add) == "add"
    
def test_basic_dir_validation():
    newdir = topdir / "newdir"
    if newdir.exists():
        newdir.rmtree()
    
    try:
        newval = context.validate_existing_directory(newdir)
        assert False, "Expected exception for directory that does not exist"
    except commands.FileError:
        pass
    
    # no exception means we're fine.
    newval = context.validate_new_directory(newdir)
    assert newval == newdir
    
    newdir.mkdir()
    
    try:
        newval = context.validate_new_directory(newdir)
        assert False, "Expected exception for existing directory"
    except commands.FileError:
        pass
    
    newval = context.validate_existing_directory(newdir)
    assert newval == newdir
    
def test_basic_file_validation():
    newdir = topdir / "newdir"
    newfile = newdir / "testfile.txt"
    if newfile.exists():
        newfile.unlink()
        
    if not newdir.exists():
        newdir.mkdir()
    
    try:
        newval = context.validate_existing_file(newfile)
        assert False, "Expected exception for file that does not exist"
    except commands.FileError:
        pass
        
    try:
        newval = context.validate_exists(newfile)
        assert False, "Expected exception for file that does not exist"
    except commands.FileError:
        pass
    
    # no exception means we're fine.
    newval = context.validate_new_file(newfile)
    assert newval == newfile
    
    newfile.write_bytes("Hi\n")
    
    try:
        newval = context.validate_new_file(newfile)
        assert False, "Expected exception for existing file"
    except commands.FileError:
        pass
    
    newval = context.validate_existing_file(newfile)
    assert newval == newfile
    newdir.rmtree()
    
def test_clone_command_checks_for_existence():
    bespin_dir = topdir / "bespin"
    if bespin_dir.exists():
        bespin_dir.rmdir()
        
    # should work when the dest directory is not there.
    clone = commands.clone(context, ["http://hg.mozilla.org/labs/bespin"])
    dest = clone.dest
    assert dest == "bespin"
    
    bespin_dir.mkdir()
    try:
        clone = commands.clone(context, ["http://hg.mozilla.org/labs/bespin"])
        assert False, "Expected FileError for existing dest directory"
    except commands.FileError:
        pass
    
    bespin_dir.rmdir()
    
def test_secure_context_keeps_user_in_working_dir():
    newdir = topdir / ".." / "newdir"
    newfile = topdir / ".." / "newfile.txt"
    assert not newdir.exists()
    assert not newfile.exists()
    
    existdir = topdir / ".."
    existfile = topdir / ".." / "setup.py"
    assert existdir.exists()
    assert existfile.exists()
        
    context = SecureContext(topdir)
    try:
        context.validate_new_directory(newdir)
        assert False, "Expected exception for going above topdir"
    except commands.SecurityError:
        pass
    
    try:
        context.validate_new_file(newfile)
        assert False, "Expected exception for going above topdir"
    except commands.SecurityError:
        pass
    
    try:
        context.validate_existing_directory(existdir)
        assert False, "Expected exception for going above topdir"
    except commands.SecurityError:
        pass
    
    try:
        context.validate_existing_file(existfile)
        assert False, "Expected exception for going above topdir"
    except commands.SecurityError:
        pass
    
    try:
        context.validate_exists(existfile)
        assert False, "Expected exception for going above topdir"
    except commands.SecurityError:
        pass

def test_clone_is_secured():
    try:
        clone = commands.clone(secure_context, 
            ["http://hg.mozilla.org/labs/bespin", topdir / ".." / "foo"])
        assert False, "Expected exception for going above topdir"
    except commands.SecurityError:
        pass
    
def test_commit_is_secured():
    try:
        commit = commands.commit(secure_context, 
            ["-m", "commit message", topdir / ".." / "foo"])
        assert False, "Expected exception for going above topdir"
    except commands.SecurityError:
        pass
    
def test_diff_is_secured():
    try:
        diff = commands.diff(secure_context, [topdir / ".." / "foo"])
        assert False, "Expected exception for going above topdir"
    except commands.SecurityError:
        pass

def test_revert_is_secured():
    try:
        revert = commands.revert(secure_context, [topdir / ".." / "foo"])
        assert False, "Expected exception for going above topdir"
    except commands.SecurityError:
        pass
        
def test_status_is_secured():
    try:
        status = commands.status(secure_context, [topdir / ".." / "foo"])
        assert False, "Expected exception for going above topdir"
    except commands.SecurityError:
        pass
        
def test_remove_is_secured():
    try:
        remove = commands.remove(secure_context, [topdir / ".." / "foo"])
        assert False, "Expected exception for going above topdir"
    except commands.SecurityError:
        pass

def test_push_is_secured():
    try:
        push = commands.push(secure_context, [topdir / ".." / "foo"])
        assert False, "Expected security error for trying to push elsewhere"
    except commands.SecurityError:
        pass
    
    try:
        push = commands.push(secure_context, ["file:///etc/passwd"])
        assert False, "Expected security error for trying to push elsewhere"
    except commands.SecurityError:
        pass
    


"""Implements the Subversion VCS dialect."""
import os
import re
import glob

import argparse

from uvc.commands import UVCError, DialectCommand, StatusOutput, BaseCommand,\
                        SimpleStringOutput
from uvc.exc import RepositoryAlreadyInitialized
from uvc import util

class SVNError(UVCError):
    """A Subversion-specific error."""
    pass
    
class OptionParser(argparse.ArgumentParser):
    def __init__(self, *args, **kw):
        kw['add_help'] = False
        argparse.ArgumentParser.__init__(self, *args, **kw)
        
    def exit(self, status=0, msg=""):
        raise SVNError(msg)

class SVNCommand(DialectCommand):
    dialect_name = "svn"

class AuthSVNCommand(SVNCommand):
    def add_auth_info(self, parts):
        auth = self.generic.auth
        if auth:
            if auth['type'] == "ssh":
                parts.insert(0, "--non-interactive")
                parts.insert(1, "--trust-server-cert")
                parts.insert(2, "")
                parts.insert(3, "")
                parts.insert(4, "")
                #parts.insert(1, "-e")
                # setting StrictHostKeyChecking to no is less than ideal...
                #parts.insert(2, "ssh -i %s -o StrictHostKeyChecking=no" % (auth['key']))
            elif auth['type'] == "password":
                parts.insert(0, "--non-interactive")
                parts.insert(1, "--username")
                parts.insert(2, auth['username'])
                parts.insert(3, "--password")
                parts.insert(4, auth['password'])
    
    def command_parts(self):
        parts = super(AuthSVNCommand, self).command_parts()
        self.add_auth_info(parts)
        return parts

class clone(AuthSVNCommand):
    reads_remote = True
    writes_remote = False

    def command_parts(self):
        parts = ["checkout", self.source_without_auth, self.dest]
        
        self.add_auth_info(parts)
            
        return parts

checkout = clone

def _commit_log_file(working_dir):
    return working_dir / ".svn_commit_messages"

class commit(SVNCommand):
    reads_remote = False
    writes_remote = False
    
    def command_parts(self):
        return []
        
    def get_command_line(self):
        return None
        
    def get_output(self):
        commit_log = _commit_log_file(self.generic.working_dir)
        if not commit_log.exists():
            content = self.generic.message + "\n\n"
        else:
            content = commit_log.text() + self.generic.message + "\n\n"
        commit_log.write_text(content)
        return SimpleStringOutput("Commit message saved. Don't forget to push to save to the remote repository!")

class diff(SVNCommand):
    reads_remote = False
    writes_remote = False

class remove(SVNCommand):
    reads_remote = False
    writes_remote = False
    
    def command_parts(self):
        parts = super(remove, self).command_parts()
        # uvc's remove command implies "force"
        parts.insert(1, "--force")
        return parts

delete = remove

_status_line_mask = re.compile("^(.)..... (.*)$")

def _get_file_list(working_dir, status="?"):
    retcode, stdout = util.run_in_directory(working_dir,
        ["svn", "status"])
    status_lines = stdout.read().split("\n")
    file_list = []
    for line in status_lines:
        m = _status_line_mask.match(line)
        if not m:
            continue
        if m.group(1) == status:
            filename = m.group(2)
            if not filename.startswith("."):
                file_list.append(m.group(2))
    
    return file_list

class add(SVNCommand):
    reads_remote = False
    writes_remote = False

    def command_parts(self):
        parts = super(add, self).command_parts()
        if not self.targets:
            parts.extend(_get_file_list(self.generic.working_dir))
        return parts

class push(AuthSVNCommand):
    reads_remote = True
    writes_remote = True
    
    def get_command_line(self):
        commit_log = _commit_log_file(self.generic.working_dir)
        if not commit_log.exists():
            return None
        return super(push, self).get_command_line()
        
    def get_output(self):
        # only called if the commit_log doesn't exist
        return SimpleStringOutput("Nothing to push. Run commit first.")
    
    def command_parts(self):
        parts = []
        self.add_auth_info(parts)
        parts.append("commit")
        parts.append("-m")
        commit_log = _commit_log_file(self.generic.working_dir)
        parts.append(commit_log.text())
        return parts
        
    def command_successful(self):
        commit_log = _commit_log_file(self.generic.working_dir)
        commit_log.unlink()

class update(AuthSVNCommand):
    reads_remote = True
    writes_remote = False

    def command_parts(self):
        parts = ["update"]
        self.add_auth_info(parts)
        return parts

class resolved(SVNCommand):
   reads_remote = False
   writes_remote = False

   def command_parts(self):
       parts = super(resolved, self).command_parts()
       parts[0] = "resolve"
       parts.insert(1, "--accept")
       parts.insert(2, "working")
       if not self.targets:
           parts.extend(_get_file_list(self.generic.working_dir, status="C"))
       return parts

class status(SVNCommand):
    reads_remote = False
    writes_remote = False

    def process_output(self, returncode, stdout):
        return StatusOutput(returncode, stdout)

class revert(SVNCommand):
    reads_remote = False
    writes_remote = False

    def command_parts(self):
        parts = super(revert, self).command_parts()
        if not self.targets:
            parts.append("-R")
            pwd = os.getcwd()
            try:
                os.chdir(self.generic.working_dir)
                parts.extend(glob.glob("*"))
            finally:
                os.chdir(pwd)
        return parts
        
class SVNNativeCommand(object):
    @classmethod
    def from_args(cls, context, initial_args):
        (options, args) = cls.parser.parse_args(initial_args)
        return cls(context, initial_args, options, args)
        
    def __init__(self, context, initial_args, options, args):
        self.context = context
        self.initial_args = initial_args
        self.options = options
        self.args = args
        
    def command_parts(self):
        return [self.__class__.__name__[4:]] + self.initial_args
        
    def get_command_line(self):
        return ["svn"] + self.command_parts()
        
class SVN_revert(SVNNativeCommand):
    reads_remote = False
    writes_remote = False
    
    parser = OptionParser()
    parser.add_argument("-h", "--help", type=bool)
    parser.add_argument("-R", type=bool)
    
class SVNDialect(object):
    
    name = "svn"
    
    def convert(self, command_object):
        """Converts to an SVN-specific command."""
        local_command = self._get_command_class(command_object.__class__.__name__)
        return local_command(command_object)
        
    def convert_class(self, command_class):
        """Converts a generic command class to its equivalent."""
        local_command_class = self._get_command_class(command_class.__name__)
        return local_command_class
        
    def _get_command_class(self, command_name):
        """Looks up an svn-specific command class."""
        local_command = globals().get(command_name)
        if not local_command:
            raise SVNError("Command cannot be converted to svn: %s" 
                            % (command_name))
        return local_command
        
    def get_dialect_command_class(self, command_name):
        """Looks up an svn-specific command class"""
        return self._get_command_class("SVN_" + command_name)
    
    def cwd_is_this_dialect(self):
        """Returns 1 if the .svn directory is here, 2 otherwise. svn
        plants directories everywhere, so if it's not in the current
        directory, it's not an svn project."""
        if os.path.isdir(".svn"):
            return 1
        return 2
    


import os
from cStringIO import StringIO

from uvc.path import path
from uvc import commands, svn, main, exc, util
from uvc.tests.util import test_context
from uvc.tests.mock import patch

StatusOutput = commands.StatusOutput

dialect = svn.SVNDialect()

topdir = path(__file__).dirname().abspath() / ".." / ".." / "testfiles"

context = None

def setup_module(module):
    global context
    if not os.path.exists(topdir):
        os.mkdir(topdir)
    context = main.Context(topdir)
    
def test_clone_command_conversion():
    generic_clone = commands.clone(context, ["http://paver.googlecode.com/svn/trunk/", 
                                    "paver"])
    svn_clone = svn.clone(generic_clone)
    svn_clone_command = " ".join(svn_clone.get_command_line())
    assert svn_clone_command == "svn checkout http://paver.googlecode.com/svn/trunk/ paver"
    
def test_convert_function():
    generic_clone = commands.clone(context, ["http://paver.googlecode.com/svn/trunk/", 
                                    "paver"])
    svn_clone = dialect.convert(generic_clone)
    svn_clone_command = " ".join(svn_clone.get_command_line())
    assert svn_clone_command == "svn checkout http://paver.googlecode.com/svn/trunk/ paver"
    
def test_convert_unknown_command():
    class Foo(object):
        pass
    
    try:
        result = dialect.convert(Foo())
        assert False, "expected SVNError for unknown command"
    except svn.SVNError:
        pass

def test_commit_command():
    commit_file = path(test_context.working_dir) / ".svn_commit_messages"
    if commit_file.exists():
        commit_file.unlink()
    
    generic_commit = commands.commit(test_context, 
                ["-m", "test message", "foo", "bar"])
    result = dialect.convert(generic_commit)
    assert result.get_command_line() == None
    assert str(result.get_output()) == "Commit message saved. Don't forget to push to save to the remote repository!"
    
    assert commit_file.exists(), "Expected commit file at " + commit_file
    assert commit_file.text() == "test message\n\n"
    
def test_push_command_before_commit():
    commit_file = path(test_context.working_dir) / ".svn_commit_messages"
    if commit_file.exists():
        commit_file.unlink()
    
    generic_push = commands.push(test_context, [])
    result = dialect.convert(generic_push)
    assert result.get_command_line() == None
    assert str(result.get_output()) == "Nothing to push. Run commit first."
    
def test_push_after_commit():
    commit_file = path(test_context.working_dir) / ".svn_commit_messages"
    if commit_file.exists():
        commit_file.unlink()
    
    generic_commit = commands.commit(test_context, 
                ["-m", "test message1", "foo", "bar"])
    result = dialect.convert(generic_commit)
    result.get_output()
    
    generic_commit = commands.commit(test_context, 
                ["-m", "test message2", "foo", "bar"])
    result = dialect.convert(generic_commit)
    result.get_output()
    
    generic_push = commands.push(test_context, [])
    result = dialect.convert(generic_push)
    assert result.get_command_line() == ["svn", "commit", "-m", 
        "test message1\n\ntest message2\n\n"]
    
def test_diff_command():
    generic_diff = commands.diff(context, [])
    result = dialect.convert(generic_diff)
    assert str(result) == "diff"
    
@patch("uvc.util.run_in_directory")
def test_add_all_files(rid):
    generic_add = commands.add(test_context, [])
    result = dialect.convert(generic_add)
    stdout = StringIO("""A      foo.txt
?      bar.txt
A      baz.txt
""")
    rid.return_value = [0, stdout]
    assert result.get_command_line() == ["svn", "add", "bar.txt"]
    assert rid.called
    
def test_revert_all_files():
    testfile = topdir / "foo.txt"
    testfile.write_text("Test text")
    try:
        generic_revert = commands.revert(context, [])
        result = dialect.convert(generic_revert)
        assert result.get_command_line() == ["svn", "revert", "-R", "foo.txt"]
    finally:
        testfile.unlink()
    
@patch("uvc.util.run_in_directory")
def test_resolve_all_files(rid):
    stdout = StringIO("""A      foo.txt
C      bar.txt
A      baz.txt
""")
    rid.return_value = [0, stdout]
    generic_resolved = commands.resolved(context, [])
    result = dialect.convert(generic_resolved)
    assert result.get_command_line() == ["svn", "resolve", 
        "--accept", "working", "bar.txt"]
    assert rid.called
    
class T(object):
    def __init__(self, command, error=False):
        self.command = command
        self.error = error
        
    @property
    def args(self):
        return ["svn"] + self.command.split(" ")
        
def test_svn_native_commands():
    tests = [
        T("revert"),
        T("revert -R"),
        T("revert --foo", error=True),
        T("revert -h"),
        T("revert --help")
    ]
    
    def test_one(t):
        try:
            command = main.convert(context, t.args, None)
        except svn.SVNError, e:
            if t.error:
                return
            else:
                assert False, "Unexpected Exception: %s" % e
                
        command_line = command.get_command_line()
        args = t.args
        assert command_line == args
    
    for t in tests:
        yield test_one, t
    


"""Utilities for testing VCS, specifically for working around not acually needing to
run a VCS."""
from functools import wraps
from cStringIO import StringIO

from uvc import main, commands
from uvc.path import path

topdir = (path(__file__).parent)

def mock_run_command(command_output, create_dir=None):
    """Monkeypatches in a new run_command function in uvc.main
    so that we can pretend about what happens when you run the
    command."""
    run_command_params = []
    def new_run_command(command, context):
        run_command_params.append(command)
        run_command_params.append(context)
        if create_dir is not None:
            working_dir = path(context.working_dir)
            (working_dir / create_dir).mkdir()
        return command.process_output(0, StringIO(command_output))
        
    def entangle(func):
        @wraps(func)
        def new_one():
            old_run_command = main.run_command
            try:
                main.run_command = new_run_command
                return func(run_command_params)
            finally:
                main.run_command = old_run_command
        return new_one
    return entangle

class TestContext(main.Context):
    def _normalize_path(self, unnorm_path):
        return super(TestContext, self)._normalize_path(topdir)

test_context = TestContext(topdir)





"""Utility functions used by uvc."""

import os
import subprocess

def run_in_directory(working_dir, command_line):
    current_dir = os.getcwd()
    os.chdir(working_dir)
    try:
        p = subprocess.Popen(command_line, 
                    stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        p.wait()
    finally:
        os.chdir(current_dir)
    
    return [p.returncode, p.stdout]
    


from setuptools import setup, find_packages

setup(
    name="uvc",
    description="The Uber Version Controller. Early release of the version control facade used by Bespin.",
    author="Kevin Dangoor",
    author_email="dangoor+uvc@gmail.com",
    version="0.3.1",
    license="MPL/GPL/LGPL",
    url="http://bitbucket.org/dangoor/uvc/",
    packages=find_packages(),
    install_requires=["argparse"],
    entry_points="""
        [console_scripts]
        uvc = uvc.main:run
    """
    
)
